@@ -1,25 +1,18 @@
 // visit unique chains in food webs.
-
 #include <R.h>
-
 #include <vector>
 #include <queue>
 #include <map>
 #include <algorithm>
-
 #include "cheddar_exception.h"
-
 typedef std::vector<int> IntVector;
-
 // Define DEBUG_TC to get lots of debug output
 //#define DEBUG_TC
-
 #ifdef DEBUG_TC
 #define DEBUG(X) { X; }
 #else
 #define DEBUG(X)
 #endif
-
 std::vector<IntVector> Adjacency(const int *adjacency, int node_count)
 {
   // Returns a C++ representation of the adjacency list in R matrix form
@@ -36,27 +29,20 @@ std::vector<IntVector> Adjacency(const int *adjacency, int node_count)
   }
   return (adj);
 }
-
 template <typename Visitor> class TrophicChains
 {
   // Enumerates every unique chain in a food web, using a breadth-first search.
-
   // A visitor is shown each unique chain. Visitors must implement a method:
   // void chain(const IntVector &);
-
   // Algorithm and first implemention by Rob Emerson.
   // Converted to use C++ and integrated into R by Lawrence Hudson.
-
   // TODO Stop when queue larger than a threshold
   // TODO Respect CTRL+C
-
 private:
   const std::vector<IntVector> &adjacency_;
   const IntVector &is_basal_;
-
   typedef std::queue<IntVector> Queue;
   Queue::size_type max_queue_;
-
 private:
 #ifdef DEBUG_TC
   void print_adjacency() const
@@ -69,7 +55,6 @@ private:
       Rprintf("\n");
     }
   }
-
   void print_path(const char *text, const IntVector &path) const
   {
     Rprintf("%s:", text);
@@ -78,7 +63,6 @@ private:
     Rprintf("\n");
   }
 #endif
-
 public:
   TrophicChains(const std::vector<IntVector> &adjacency,
                 const IntVector &is_basal,
@@ -88,13 +72,11 @@ public:
          max_queue_(max_queue)
   {
   }
-
   void visit(Visitor &visitor) const
   {
     // Visits unique trophic chains the network represented by adjancency.
     DEBUG(Rprintf("DUMP GRAPH\n"));
     DEBUG(print_adjacency());
-
     bool queue_warning = false;
     for(int n = 0; n < adjacency_.size(); ++n)
     {
@@ -102,18 +84,14 @@ public:
                                               // cannot start a chain
       if(is_basal_[n] == 0) continue;  // Node n has in-edges, so it cannot
                                        // start a chain
-
       IntVector path(1, n);
       DEBUG(print_path("INITIAL PATH", path));
-
       Queue queue;
       queue.push(path);
-
       while(!queue.empty())
       {
         path = queue.front();
         queue.pop();
-
         R_ProcessEvents();
         if(max_queue_>0 && !queue_warning && queue.size()>max_queue_/2)
         {
@@ -126,11 +104,8 @@ public:
           throw CheddarException("Unable to compute paths - see the help for "
                                  "TrophicChains for more information.");
         }
-
         int m = path.back();
-
         DEBUG(Rprintf("AT NODE [%d]\n", m));
-
         if(adjacency_[m].size() == 0)
         {
           DEBUG(print_path("", path));
@@ -139,7 +114,6 @@ public:
         else
         {
           bool cycle = true;
-
           for(int j=0; j<adjacency_[m].size(); ++j)
           {
             bool found = false;
@@ -151,18 +125,15 @@ public:
                 break;
               }
             }
-
             if(!found)
             {
               path.push_back(adjacency_[m][j]);
               DEBUG(print_path("EXTEND PATH", path));
               queue.push(path);
               path.pop_back();
-
               cycle = false;
             }
           }
-
           if(cycle)
           {
             DEBUG(print_path("", path));
@@ -173,18 +144,14 @@ public:
     }
   }
 };
-
 class CollectChainsVisitor
 {
   // Collects chains
-
 private:
     int *chains_;     // An array of ncols_ * nrows_
     int ncols_;
     int nrows_;
-
     int n_chains_found_;
-
 public:
     CollectChainsVisitor(int *chains,  int ncols, int nrows) :
       chains_(chains),
@@ -193,14 +160,13 @@ public:
       n_chains_found_(0)
   {
   }
-
   void chain(const IntVector &path)
   {
     if(n_chains_found_ <= ncols_ && path.size()<=IntVector::size_type(nrows_))
     {
       const IntVector::size_type path_length(path.size());
-      memcpy(chains_ + n_chains_found_ * nrows_, &path[0],
-             path_length*sizeof(IntVector::value_type));
+      std::memcpy(chains_ + n_chains_found_ * nrows_, &path[0],
+                  path_length*sizeof(IntVector::value_type));
       n_chains_found_ += 1;
     }
     else
@@ -213,7 +179,6 @@ public:
     }
   }
 };
-
 class PrintChainsVisitor
 {
   // Prints chains to stdout in the same format as Rob's solution8
@@ -229,14 +194,12 @@ public:
     Rprintf("\n");
   }
 };
-
 class CollectChainLengthsVisitor
 {
   // Records the number of chains and the length of the longest chain.
 public:
   int longest_;
   int n_chains_;
-
 public:
   CollectChainLengthsVisitor(bool test_overflow=false) : longest_(0),
                                                          n_chains_(0)
@@ -246,7 +209,6 @@ public:
       n_chains_ = INT_MAX;
     }
   }
-
   void chain(const IntVector &path)
   {
     longest_ = std::max(longest_, int(path.size()));
@@ -260,7 +222,6 @@ public:
     }
   }
 };
-
 class ChainStatsVisitor
 {
   // Records the length of each chains and the number of times that each node
@@ -268,9 +229,7 @@ class ChainStatsVisitor
 public:
   typedef std::vector<IntVector> CountVector;
   CountVector counts_;
-
   IntVector chain_lengths_;    // A vector of chain lengths
-
 public:
   ChainStatsVisitor(int nodes, int longest) : counts_(nodes)
   {
@@ -279,18 +238,15 @@ public:
       it->resize(longest);
     }
   }
-
   void chain(const IntVector &path)
   {
     chain_lengths_.push_back(path.size()-1);
-
     for(IntVector::size_type position=0; position<path.size(); ++position)
     {
       IntVector::const_reference node = path[position];
       counts_[node][position] += 1;
     }
   }
-
 #ifdef DEBUG_TC
   void print() const
   {
@@ -308,10 +264,8 @@ public:
   }
 #endif
 };
-
 extern "C"
 {
-
 void trophic_chains_size(const int *adjacency,
                          const int *adjacency_length,
                          const int *is_basal,
@@ -323,38 +277,27 @@ void trophic_chains_size(const int *adjacency,
                          int *status)
 {
   /* WARNING: Nested returns */
-
   /* Enumerates every unique trophic path in a directed graph. Output is
      written to chains.
-
      In params:
      adjacency:   a matrix of node_count rows. First column is the number of
                   consumers of that row. Subsequent columns are ids of
                   consumers.
-
 adjacency_length: the number of ints in adjacency
-
       is_basal:   an array of length node_count. 1 for nodes that have no
                   resources, 0 otherwise. Chains start only with nodes which
                   have a 1 in is_basal.
-
     node_count:   the number of nodes in the network.
-
  test_overflow:   if non-zero then check the overflow case for counting chains.
-
      max_queue:   the maximum alowabe size of the queue used to compute chains.
                   0 indicates no limit.
-
     Out params:
       n_chains:   the number of unique chains found.
-
        longest:   the number of nodes in the longest chain found.
-
         status: -1 - unexpected error.
                  0 - normal exit.
                  1 - problem with one of the parameters.
 */
-
   /* Quick and dirty parameter checks */
   if(0==adjacency || 0==adjacency_length || *adjacency_length<1 ||
      0==is_basal || 0==node_count || *node_count<1 || 0==test_overflow ||
@@ -368,9 +311,7 @@ adjacency_length: the number of ints in adjacency
     /* WARNING: Nested return */
     return;
   }
-
   *status = -1;    // Default to an error status code
-
   try
   {
     std::vector<IntVector> adj = Adjacency(adjacency, *node_count);
@@ -378,10 +319,8 @@ adjacency_length: the number of ints in adjacency
     TrophicChains<CollectChainLengthsVisitor> worker(adj, basal, *max_queue);
     CollectChainLengthsVisitor visitor(0!=*test_overflow);
     worker.visit(visitor);
-
     *n_chains = visitor.n_chains_;
     *longest = visitor.longest_;
-
     // Normal exit
     *status = 0;
   }
@@ -394,7 +333,6 @@ adjacency_length: the number of ints in adjacency
     REprintf("Unexpected error in trophic_chains_size\n");
   }
 }
-
 void print_chains(const int *adjacency,
                   const int *adjacency_length,
                   const int *is_basal,
@@ -403,36 +341,26 @@ void print_chains(const int *adjacency,
                   int *status)
 {
   /* WARNING: Nested returns */
-
   /* Enumerates every unique trophic path in a directed graph. Output is
      written to chains.
-
      In params:
      adjacency:   a matrix of node_count rows. First column is the number of
                   consumers of that row. Subsequent columns are ids of
                   consumers.
-
 adjacency_length: the number of ints in adjacency
-
       is_basal:   an array of length node_count. 1 for nodes that have no
                   resources, 0 otherwise. Chains start only with nodes which
                   have a 1 in is_basal.
-
     node_count:   the number of nodes in the network.
-
          ncols:   the number of columns in chains.
-
          nrows:   the number of rows in chains.
-
      max_queue:   the maximum alowabe size of the queue used to compute chains.
                   0 indicates no limit.
-
      Out params:
         status: -1 - unexpected error.
                  0 - normal exit.
                  1 - problem with one of the parameters.
   */
-
   /* Quick and dirty parameter checks */
   if(0==adjacency || 0==adjacency_length || *adjacency_length<1 ||
      0==is_basal || 0==node_count || 0==max_queue || *max_queue<0 ||
@@ -445,9 +373,7 @@ adjacency_length: the number of ints in adjacency
     /* WARNING: Nested return */
     return;
   }
-
   *status = -1;    // Default to an error status code
-
   try
   {
     std::vector<IntVector> adj = Adjacency(adjacency, *node_count);
@@ -455,7 +381,6 @@ adjacency_length: the number of ints in adjacency
     TrophicChains<PrintChainsVisitor> worker(adj, basal, *max_queue);
     PrintChainsVisitor visitor;
     worker.visit(visitor);
-
     // Normal exit
     *status = 0;
   }
@@ -468,7 +393,6 @@ adjacency_length: the number of ints in adjacency
     REprintf("Unexpected error in print_chains\n");
   }
 }
-
 void trophic_chains(const int *adjacency,
                     const int *adjacency_length,
                     const int *is_basal,
@@ -480,39 +404,28 @@ void trophic_chains(const int *adjacency,
                     int *status)
 {
   /* WARNING: Nested returns */
-
   /* Enumerates every unique trophic path in a directed graph. Output is
      written to chains.
-
      In params:
      adjacency:   a matrix of node_count rows. First column is the number of
                   consumers of that row. Subsequent columns are ids of
                   consumers.
-
 adjacency_length: the number of ints in adjacency
-
       is_basal:   an array of length node_count. 1 for nodes that have no
                   resources, 0 otherwise. Chains start only with nodes which
                   have a 1 in is_basal.
-
     node_count:   the number of nodes in the network.
-
          ncols:   the number of columns in chains.
-
          nrows:   the number of rows in chains.
-
      max_queue:   the maximum alowabe size of the queue used to compute chains.
                   0 indicates no limit.
-
     Out params:
         chains:   a matrix of node_count rows and max_chains columns. Each
                   unique chain will be written to this matrix.
-
           status: -1 - unexpected error.
                    0 - normal exit.
                    1 - problem with one of the parameters.
   */
-
   /* Quick and dirty parameter checks */
   if(0==adjacency || 0==adjacency_length || *adjacency_length<1 ||
      0==is_basal || 0==node_count || *node_count<1 ||
@@ -527,9 +440,7 @@ adjacency_length: the number of ints in adjacency
     /* WARNING: Nested return */
     return;
   }
-
   *status = -1;    // Default to an error status code
-
   try
   {
     std::vector<IntVector> adj = Adjacency(adjacency, *node_count);
@@ -537,7 +448,6 @@ adjacency_length: the number of ints in adjacency
     TrophicChains<CollectChainsVisitor> worker(adj, basal, *max_queue);
     CollectChainsVisitor visitor(chains, *ncols, *nrows);
     worker.visit(visitor);
-
     // Normal exit
     *status = 0;
   }
@@ -550,7 +460,6 @@ adjacency_length: the number of ints in adjacency
     REprintf("Unexpected error in trophic_chains\n");
   }
 }
-
 void trophic_chains_stats(const int *adjacency,
                           const int *adjacency_length,
                           const int *is_basal,
@@ -563,43 +472,31 @@ void trophic_chains_stats(const int *adjacency,
                           int *status)
 {
   /* WARNING: Nested returns */
-
   /* Enumerates every unique trophic path in a directed graph. Outputs are the
      mean, minimum and maximum position of each node in every chain.
-
      In params:
      adjacency:   a matrix of node_count rows. First column is the number of
                   consumers of that row. Subsequent columns are ids of
                   consumers.
-
 adjacency_length: the number of ints in adjacency
-
       is_basal:   an array of length node_count. 1 for nodes that have no
                   resources, 0 otherwise. Chains start only with nodes which
                   have a 1 in is_basal.
-
     node_count:   the number of nodes in the network.
-
       n_chains:   the number of chains.
-
        longest:   the number of nodes in the longest chain.
-
      max_queue:   the maximum alowabe size of the queue used to compute chains.
                   0 indicates no limit.
-
     Out params:
 node_pos_counts:  an array of n_chains * longest integers. Will contain
                   counts of the number of times each node appears at a given
                   position in the chain.
-
   chain_lengths:  an array of n_chains integers. Will contain the length of
                   each chain.
-
         status: -1 - unexpected error.
                  0 - normal exit.
                  1 - problem with one of the parameters.
   */
-
   /* Quick and dirty parameter checks */
   if(0==adjacency || 0==adjacency_length || *adjacency_length<1 ||
      0==is_basal || 0==node_count || *node_count<1 ||
@@ -614,9 +511,7 @@ node_pos_counts:  an array of n_chains * longest integers. Will contain
     /* WARNING: Nested return */
     return;
   }
-
   *status = -1;    // Default to an error status code
-
   try
   {
     std::vector<IntVector> adj = Adjacency(adjacency, *node_count);
@@ -624,7 +519,6 @@ node_pos_counts:  an array of n_chains * longest integers. Will contain
     TrophicChains<ChainStatsVisitor> worker(adj, basal, *max_queue);
     ChainStatsVisitor visitor(*node_count, *longest);
     worker.visit(visitor);
-
     if(sizeof(IntVector::value_type)!=sizeof(*chain_lengths) ||
           sizeof(IntVector::value_type)!=sizeof(*node_pos_counts))
     {
@@ -640,7 +534,7 @@ node_pos_counts:  an array of n_chains * longest integers. Will contain
     }
     else
     {
-      memcpy(chain_lengths, &visitor.chain_lengths_[0], sizeof(int) * *n_chains);
+      std::memcpy(chain_lengths, &visitor.chain_lengths_[0], sizeof(int) * *n_chains);
       for(ChainStatsVisitor::CountVector::size_type node=0;
           node<visitor.counts_.size(); ++node)
       {
@@ -651,7 +545,7 @@ node_pos_counts:  an array of n_chains * longest integers. Will contain
         }
         else
         {
-          memcpy(node_pos_counts + node * *longest, &v[0],
+          std::memcpy(node_pos_counts + node * *longest, &v[0],
                  sizeof(int) * *longest);
         }
       }
@@ -668,5 +562,4 @@ node_pos_counts:  an array of n_chains * longest integers. Will contain
     REprintf("Unexpected error in trophic_chains_stats\n");
   }
 }
-
 }   // extern "c"
